Class: Snow::Vec3Array

Inherits:
Data
  • Object
show all
Includes:
ArrayMarshalSupport, ArraySupport, FiddlePointerSupport, InspectSupport
Defined in:
lib/snow-math/vec3.rb,
lib/snow-math/ptr.rb,
lib/snow-math/to_a.rb,
lib/snow-math/inspect.rb,
lib/snow-math/marshal.rb,
ext/snow-math/snow-math.c

Overview

A contiguous array of Vec3s. Allocated as a single block of memory so that it can easily be passed back to C libraries (like OpenGL) and to aid with cache locality.

Useful for storing vertex data such as positions, normals, and so on.

Class Method Summary collapse

Instance Method Summary collapse

Methods included from ArrayMarshalSupport

#_dump, included

Methods included from InspectSupport

#inspect

Methods included from ArraySupport

#each, #map, #map!, #to_a

Methods included from FiddlePointerSupport

#to_ptr

Class Method Details

.new(sm_length_or_copy) ⇒ Object Also known as: []

In the first form, a new typed array of Vec3 elements is allocated and returned. In the second form, a copy of a typed array of Vec3 objects is made and returned. Copied arrays do not share data.

call-seq:

new(size)       -> new vec3_array
new(vec3_array) -> copy of vec3_array


354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
# File 'ext/snow-math/snow-math.c', line 354

static VALUE sm_vec3_array_new(VALUE sm_self, VALUE sm_length_or_copy)
{
  size_t length = 0;
  vec3_t *arr;
  VALUE sm_type_array;
  int copy_array = 0;
  if ((copy_array = SM_IS_A(sm_length_or_copy, vec3_array))) {
    length = NUM2SIZET(sm_mathtype_array_length(sm_length_or_copy));
  } else {
    length = NUM2SIZET(sm_length_or_copy);
  }
  if (length <= 0) {
    return Qnil;
  }
  arr = ALLOC_N(vec3_t, length);
  if (copy_array) {
    const vec3_t *source;
    Data_Get_Struct(sm_length_or_copy, vec3_t, source);
    MEMCPY(arr, source, vec3_t, length);
    sm_length_or_copy = sm_mathtype_array_length(sm_length_or_copy);
    sm_self = rb_obj_class(sm_length_or_copy);
  }
  sm_type_array = Data_Wrap_Struct(sm_self, 0, free, arr);
  rb_ivar_set(sm_type_array, kRB_IVAR_MATHARRAY_LENGTH, sm_length_or_copy);
  rb_ivar_set(sm_type_array, kRB_IVAR_MATHARRAY_CACHE, rb_ary_new2((long)length));
  rb_obj_call_init(sm_type_array, 0, 0);
  return sm_type_array;
}

Instance Method Details

#addressObject

Returns the memory address of the object.

call-seq: address -> fixnum



6683
6684
6685
6686
6687
6688
# File 'ext/snow-math/snow-math.c', line 6683

static VALUE sm_get_address(VALUE sm_self)
{
  void *data_ptr = NULL;
  Data_Get_Struct(sm_self, void, data_ptr);
  return ULL2NUM((unsigned long long)data_ptr);
}

#dupObject Also known as: clone

Duplicates the Vec3Array and returns it.

call-seq: dup -> new vec3_array



143
144
145
# File 'lib/snow-math/to_a.rb', line 143

def dup
  self.class.new(self)
end

#fetch(sm_index) ⇒ Object Also known as: []

Fetches a Vec3 from the array at the index and returns it. The returned Vec3 may be a cached object. In all cases, values returned from a typed array are associated with the memory of the array and not given their own memory. So, modifying a Vec3 fetched from an array modifies the array’s data.

As a result, objects returned by a Vec3Array should not be considered thread-safe, nor should manipulating a Vec3Array be considered thread-safe either. If you want to work with data returned from an array without altering the array data, you should call Vec3#dup or Vec3#copy to get a new Vec3 with a copy of the array object’s data.

call-seq: fetch(index) -> vec3



445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
# File 'ext/snow-math/snow-math.c', line 445

static VALUE sm_vec3_array_fetch(VALUE sm_self, VALUE sm_index)
{
  vec3_t *arr;
  size_t length = NUM2SIZET(sm_mathtype_array_length(sm_self));
  size_t index = NUM2SIZET(sm_index);
  VALUE sm_inner;
  VALUE sm_cache;
  if (index >= length) {
    rb_raise(rb_eRangeError,
      "Index %zu out of bounds for array with length %zu",
      index, length);
  }

  sm_cache = rb_ivar_get(sm_self, kRB_IVAR_MATHARRAY_CACHE);
  if (!RTEST(sm_cache)) {
    rb_raise(rb_eRuntimeError, "No cache available");
  }
  sm_inner = rb_ary_entry(sm_cache, (long)index);

  if (!RTEST(sm_inner)) {
    /* No cached value, create one. */
    Data_Get_Struct(sm_self, vec3_t, arr);
    sm_inner = Data_Wrap_Struct(s_sm_vec3_klass, 0, 0, arr[index]);
    rb_ivar_set(sm_inner, kRB_IVAR_MATHARRAY_SOURCE, sm_self);
    /* Store the Vec3 in the cache */
    rb_ary_store(sm_cache, (long)index, sm_inner);
  }

  return sm_inner;
}

#lengthObject

Returns the array’s length.

call-seq: length -> fixnum



84
85
86
87
# File 'ext/snow-math/snow-math.c', line 84

static VALUE sm_mathtype_array_length(VALUE sm_self)
{
  return rb_ivar_get(sm_self, kRB_IVAR_MATHARRAY_LENGTH);
}

#resize!(sm_new_length) ⇒ Object

Resizes the array to new_length and returns self.

If resizing to a length smaller than the previous length, excess array elements are discarded and the array is truncated. Otherwise, when resizing the array to a greater length than previous, new elements in the array will contain garbage values.

If new_length is equal to self.length, the call does nothing to the array.

Attempting to resize an array to a new length of zero or less will raise a RangeError. Do not try to resize arrays to zero or less. Do not be that person.

call-seq:

resize!(new_length) -> self


402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
# File 'ext/snow-math/snow-math.c', line 402

static VALUE sm_vec3_array_resize(VALUE sm_self, VALUE sm_new_length)
{
  size_t new_length;
  size_t old_length;

  rb_check_frozen(sm_self);

  old_length = NUM2SIZET(sm_mathtype_array_length(sm_self));
  new_length = NUM2SIZET(sm_new_length);

  if (old_length == new_length) {
    /* No change, done */
    return sm_self;
  } else if (new_length < 1) {
    /* Someone decided to be that person. */
    rb_raise(rb_eRangeError,
      "Cannot resize array to length less than or equal to 0.");
    return sm_self;
  }

  REALLOC_N(RDATA(sm_self)->data, vec3_t, new_length);
  rb_ivar_set(sm_self, kRB_IVAR_MATHARRAY_LENGTH, sm_new_length);
  rb_ary_clear(rb_ivar_get(sm_self, kRB_IVAR_MATHARRAY_CACHE));

  return sm_self;
}

#sizeObject

Returns the length of the array.

call-seq: length -> fixnum



523
524
525
526
527
# File 'ext/snow-math/snow-math.c', line 523

static VALUE sm_vec3_array_size(VALUE sm_self)
{
  size_t length = NUM2SIZET(sm_mathtype_array_length(sm_self));
  return SIZET2NUM(length * sizeof(vec3_t));
}

#store(sm_index, sm_value) ⇒ Object Also known as: []=

Stores a Vec3 at the given index. If the provided Vec3 is a member of the array and stored at the index, then no copy is done, otherwise the Vec3 is copied to the array.

call-seq: store(index, value) -> value



485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
# File 'ext/snow-math/snow-math.c', line 485

static VALUE sm_vec3_array_store(VALUE sm_self, VALUE sm_index, VALUE sm_value)
{
  vec3_t *arr;
  vec3_t *value;
  size_t length = NUM2SIZET(sm_mathtype_array_length(sm_self));
  size_t index = NUM2SIZET(sm_index);

  rb_check_frozen(sm_self);

  if (index >= length) {
    rb_raise(rb_eRangeError,
      "Index %zu out of bounds for array with length %zu",
      index, length);
  } else if (!SM_IS_A(sm_value, vec3) && !SM_IS_A(sm_value, vec4) && !SM_IS_A(sm_value, quat)) {
    rb_raise(rb_eTypeError,
      "Invalid value to store: expected Vec3, Vec4, or Quat, got %s",
      rb_obj_classname(sm_value));
  }

  Data_Get_Struct(sm_self, vec3_t, arr);
  value = sm_unwrap_vec3(sm_value, NULL);

  if (value == &arr[index]) {
    /* The object's part of the array, don't bother copying */
    return sm_value;
  }

  vec3_copy(*value, arr[index]);
  return sm_value;
}